# -*- coding: utf-8 -*-
# Copyright 2009-2015, Red Hat, Inc.
# License: GPL-2.0+ <http://spdx.org/licenses/GPL-2.0+>
# See the LICENSE file for more details on Licensing

import pytest
from mock import Mock, MagicMock
from dingus import Dingus

from libtaskotron import vm
from libtaskotron import exceptions as exc

from testcloud import image, instance
from testcloud.exceptions import TestcloudImageError, TestcloudInstanceError


class TestvmImagePrepare(object):
    '''Test libtaskotron.vm._prepare_image'''

    def setup_method(self, method):
        self.ref_uuid = '1234-5678-9012'

    def should_raise_on_failure(self, monkeypatch):
        stub_image = Mock(side_effect=TestcloudImageError())

        monkeypatch.setattr(image, 'Image', stub_image)

        test_vm = vm.TestCloudMachine(self.ref_uuid)

        with pytest.raises(exc.TaskotronRemoteError):
            test_vm._prepare_image()

    def should_behave_on_success(self, monkeypatch):
        stub_image = MagicMock()

        monkeypatch.setattr(image, 'Image', stub_image)

        test_vm = vm.TestCloudMachine(self.ref_uuid)

        test_vm._prepare_image()


class TestvmInstancePrepare(object):
    '''Test libtaskotron.vm._prepare_instance'''

    def setup_method(self, method):
        self.ref_uuid = '1234-5678-9012'

    def should_make_proper_calls(self, monkeypatch):
        stub_image = Mock()
        stub_instance = MagicMock()
        stub_instanceclass = Mock(return_value=stub_instance)

        monkeypatch.setattr(instance, 'Instance', stub_instanceclass)

        test_vm = vm.TestCloudMachine(self.ref_uuid)

        test_vm._prepare_instance(stub_image)

        assert stub_instance.method_calls == [('prepare',), ('spawn_vm',)]


class TestvmStop(object):
    '''Test libtaskotron.vm._stop_instance'''

    def setup_method(self, method):
        self.ref_uuid = '1234-5678-9012'

    def should_bequiet_on_success(self):
        stub_instance = Mock()

        test_vm = vm.TestCloudMachine(self.ref_uuid)

        test_vm._stop_instance(stub_instance)

    def should_raise_on_failure(self):
        stub_instance = Mock()
        stub_instance.stop.side_effect = TestcloudInstanceError()

        test_vm = vm.TestCloudMachine(self.ref_uuid)

        with pytest.raises(exc.TaskotronRemoteError):
            test_vm._stop_instance(stub_instance)


class TestvmDestroy(object):
    '''Tests for the libtaskotron.vm.destroy() method'''

    def setup_method(self, method):
        self.ref_uuid = '1234-5678-9012'

    def should_bequiet_on_success(self):
        stub_instance = Mock()

        test_vm = vm.TestCloudMachine(self.ref_uuid)

        test_vm._destroy_instance(stub_instance)

    def should_raise_on_failure(self):
        stub_instance = Mock()
        stub_instance.destroy.side_effect = TestcloudInstanceError()

        test_vm = vm.TestCloudMachine(self.ref_uuid)

        with pytest.raises(exc.TaskotronRemoteError):
            test_vm._destroy_instance(stub_instance)


class TestvmExistingInstances(object):
    '''Test the possible outcomes of checking for existing instances. Exception
    should be thrown if the should_exist arg doesn't match "reality".'''

    def setup_method(self, method):
        self.ref_uuid = '1234-5678-9012'

    def should_raise_existing_expecting_novm(self, monkeypatch):
        stub_instance = Mock(return_value=None)

        monkeypatch.setattr(instance, 'find_instance', stub_instance)

        test_vm = vm.TestCloudMachine(self.ref_uuid)

        with pytest.raises(exc.TaskotronRemoteError):
            test_vm._check_existing_instance(True)

    def should_return_none_expecting_novm(self, monkeypatch):
        stub_instance = Mock(return_value=None)

        monkeypatch.setattr(instance, 'find_instance', stub_instance)

        test_vm = vm.TestCloudMachine(self.ref_uuid)

        test_instance = test_vm._check_existing_instance(False)

        assert test_instance is None

    def should_return_instance_found_expecting_vm(self, monkeypatch):
        stub_found_instance = Mock()
        stub_instance = Mock(return_value=stub_found_instance)

        monkeypatch.setattr(instance, 'find_instance', stub_instance)

        test_vm = vm.TestCloudMachine(self.ref_uuid)

        test_instance = test_vm._check_existing_instance(True)

        assert test_instance == stub_found_instance

    def should_raise_noexisting_expecting_vm(self, monkeypatch):
        stub_found_instance = Mock()
        stub_instance = Mock(return_value=stub_found_instance)

        monkeypatch.setattr(instance, 'find_instance', stub_instance)

        test_vm = vm.TestCloudMachine(self.ref_uuid)

        with pytest.raises(exc.TaskotronRemoteError):
            test_vm._check_existing_instance(False)


@pytest.mark.usefixtures("setup")
class TestImageLatest:

    @pytest.fixture
    def setup(self, monkeypatch):
        self.images = ['20151008_04-f23-cloud.qcow2',
                       'ubuntu-fabulous-fedora.qcow2',
                       '20151008_02-f23-minimal.qcow2',
                       '20151105_01-server.qcow2',
                       'el_capitan.iso',
                       '20151008_01-server.qcow2',
                       '20151007_05-f23-minimal.qcow2',
                       'windows10.raw',
                       '20151008_03-f23-minimal.qcow2']
        self.stub_list_images = Dingus()
        monkeypatch.setattr(vm.Image, 'get_all_filenames', self.stub_list_images)

    def test_dir_populated(self):
        self.stub_list_images.return_value = self.images

        assert vm.Image.get_latest_metadata('f23', 'minimal', '')['filename'] ==\
            '20151008_03-f23-minimal.qcow2'

    def test_dir_populated_single(self):
        self.stub_list_images.return_value = self.images

        assert vm.Image.get_latest_metadata('f23', 'cloud', '')['filename'] ==\
            '20151008_04-f23-cloud.qcow2'

    def test_dir_empty(self):
        self.stub_list_images.return_value = []

        assert vm.Image.get_latest_metadata('f23', 'minimal', '') is None

    def test_no_such_image(self):
        self.stub_list_images.return_value = self.images

        assert vm.Image.get_latest_metadata('f69', 'ultimate', '') is None
